
/*====================================

   Module: TREECTRL
   Header-file (.HPP)
   (C) Copyright Greg Arakelian 1994,1995.
   All Rights Reserved.

  ===================================*/

#ifndef __TREECTRL_HPP
#define __TREECTRL_HPP


// ====================================
//  Include Files
// ====================================

//
// Assumes OWL already included
//

//
// Standard library
//

#ifndef __AUXDATA_HPP
#include "auxdata.hpp"
#endif

//
// Resources
//

#ifndef __TREECTRL_RH
#include "treectrl.rh"
#endif


// ====================================
//  TTreeItem CLASS DEFINITION
// ====================================

class TTreeListBox;

class TTreeItem
{
protected:
  int LevelNo;				// indentation level (-1=tree)
  int ImageNo;				// default image number
  int SelectedImageNo;			// selected image number
  char *Label;				// pointer to actual text
  bool Selected;			// true if item is selected
  bool Selectable;			// true if item can be selected
  bool Expanded;		       	// true if item is open
  int TextWidth;			// width of text label (pixels)
  TTreeItem *Parent;			// pointer to parent
  TTreeItem *Sibling;			// pointer to next sibling
  TTreeItem *FirstChild;		// pointer to first child
  TTreeItem *LastChild;			// pointer to last child
  void AddChild (TTreeItem *child);	// add child to linked list
  void DeleteChildren();		// delete list of kids
public:
  TTreeItem (TTreeItem *parent, char *label, int imageNo=-1, int selectedImageNo=-1, bool selectable=false);
  ~TTreeItem();
  virtual void Collapse (int depth=0);
  static void TTreeItem::DrawItem (DRAWITEMSTRUCT far &drawItem,
    int indent, bool linesAtRoot, bool hasButtons, bool hasLines,
    TBitmap *imageList, TBitmap *imageMask, TSize imageSize,
    TFont *treeFont, int fontHeight);
  virtual bool Expand (int depth=0);
  inline TTreeItem *GetFirstChild() { return FirstChild; }
  virtual int GetImageIndex (bool isSelected=false);
  inline char *GetLabel() { return Label; }
  inline int GetLevelNo() { return LevelNo; }
  inline TTreeItem *GetSibling() { return Sibling; }
  DWORD GetSiblingBits();
  inline TTreeItem *GetParent() { return Parent; }
  inline int GetTextWidth() { return TextWidth; }
  inline bool HasChildren() { return FirstChild!=NULL; }
  inline bool HasSibling() { return Sibling!=NULL; }
  bool IsCollapsed (int depth=0);
  inline bool IsExpanded (int depth=0);
  inline bool IsSelectable() { return Selectable; }
  inline bool IsSelected() { return Selected; }
  virtual bool Select() { return Selected=Selectable; }
  inline void SetImageNo (int imageNo) { ImageNo=imageNo; }
  inline void SetLabel (char *label) { free(Label); Label=strdup(label); }
  inline void SetTextWidth (int textWidth) { TextWidth=textWidth; }
  inline void SetSelectedImageNo (int selectedImageNo) { SelectedImageNo=selectedImageNo; }
  virtual void Unselect() { Selected=false; }
};


// ====================================
//  TTreeListBox CLASS DEFINITION
// ====================================

//
// Hit-testing values for TTreeListBox (TCHT_xxx)
//

#define TCHT_NOWHERE		0x0000
#define TCHT_ERROR		0x0001
#define TCHT_ONITEMBUTTON	0x0100
#define TCHT_ONITEMIMAGE	0x0200
#define TCHT_ONITEMLABEL	0x0400
#define TCHT_ONITEMRIGHT	0x0800
#define TCHT_ONITEMINDENT	0x1000	// indenture level added to constant


class TTreeListBox : public TListBox
{
protected:
  TBitmap *ImageList;			// image bitmap
  TBitmap *ImageMask;			// image mask
  TColor ImageBkColor;			// image background color
  TSize ImageSize;			// size of each bitmap cel
  bool HasButtons;			// true to display plus (+) and minus (-) buttons
  bool HasLines;			// true if we should draw lines to show hiearachy
  bool LinesAtRoot;			// true to display lines to link items at tree
  int Indent;				// amount of indentation for child item
  TFont *TreeFont;			// font used to label items
  int FontHeight;			// equivalent to logFont.lfHeight
  int LineHeight;			// height of owner draw items (>=FontHeight)
  TTreeItem *Tree;			// pointer to root tree item
  bool OnlyWhenClipped;			// TitleTips only when clipped?
  TTreeItem *LastTitleTip;		// pointer to last TitleTip'd item
  void CmCommand (WPARAM commandId);
  void CmCommandEnable (TCommandEnabler &tce);
  void CollapseChildren (TTreeItem *treeItem, int Index);
  virtual bool DefaultAction (bool canCollapse=true, bool shouldAdvance=false);
  virtual void DrawItem (DRAWITEMSTRUCT far &drawInfo);
  void EvChar (uint key, uint repeatCount, uint flags);
  uint EvGetDlgCode (MSG far *msg);
  void EvKeyDown (uint key, uint repeatCount, uint flags);
  void EvLButtonDblClk (uint modKeys, TPoint &point);
  void EvLButtonDown (uint modKeys, TPoint &point);
  void EvMouseMove (uint modKeys, TPoint &point);
  void EvRButtonDown (uint modKeys, TPoint &point);
  void EvSize (uint sizeType, TSize &size);
  void ExpandChildren (TTreeItem *treeItem, int index);
  int HitTest (int &hitIndex, TPoint point);
  virtual bool GetContextMenu (TContextMenu &contextMenu);
  int GetIndexFromPoint (TPoint point);
  int InsertChildren (TTreeItem *treeItem, int atIndex, int depth=-1);
  virtual void MeasureItem (MEASUREITEMSTRUCT far &measureInfo);
  void SetupWindow();
  void ShowChildren (int currentIndex, int numChildren);
public:
  TTreeListBox (TWindow *parent, int id,
    int x, int y, int w, int h, TTreeItem *tree, bool hasLines=true,
    TDib *imageListDib=0, TSize imageSize=TPoint(16,16), TModule *module=0);
  TTreeListBox (TWindow *parent, int resourceId,
    TTreeItem *tree, bool hasLines=true, TDib *imageListDib=0,
    TSize imageSize=TPoint(16,16), TModule *module=0);
  ~TTreeListBox();
  void CloseAll();
  TTreeItem *GetSelectedItem();
  void SetSelection (int index);
  DECLARE_RESPONSE_TABLE(TTreeListBox);
};


// ====================================
//  TTreeComboBox CLASS DEFINITION
// ====================================

class TTreeComboBox : public TComboBox
{
protected:
  TBitmap *ImageList;			// image bitmaps
  TBitmap *ImageMask;			// image masks
  TColor ImageBkColor;			// image background color
  TSize ImageSize;			// size of each bitmap cel
  int Indent;				// amount of indentation for child item
  TFont *TreeFont;			// font used to label items
  int FontHeight;			// equivalent to logFont.lfHeight
  int LineHeight;			// height of owner draw items (>=FontHeight)
  TTreeItem *Tree;			// pointer to tree item
  virtual void DrawItem (DRAWITEMSTRUCT far &drawInfo);
  int InsertChildren (TTreeItem *treeItem, int atIndex);
  virtual void MeasureItem (MEASUREITEMSTRUCT far &measureInfo);
  void SetupWindow();
public:
  TTreeComboBox (TWindow *parent, int id,
    int x, int y, int w, int h, TTreeItem *tree,
    TDib *imageListDib=0, TSize imageSize=TPoint(16,16), TModule *module=0);
  ~TTreeComboBox();
  TTreeItem *GetSelectedItem();
  void SetSelection (int index);
  DECLARE_RESPONSE_TABLE(TTreeComboBox);
};


#endif

